#include "ExSortFilterModel.h"

#include <QStringList>
#include <QDateTime>
#include <QDebug>

#define EXSFM_FORMAT_DATE "yyyy.MM.dd"
#define EXSFM_FORMAT_TIME "hh:mm:ss"
#define EXSFM_FORMAT_DATETIME "yyyy.MM.dd hh:mm:ss"

/*!
	\class ExSortFilterModel

	\ingroup Qt Gui extension
	\brief Модель выборки-фильтрации с расширенными возможностями.
	\mainclass

	ExSortFilterModel позволяет осуществлять выборку данных из моделей по фильтрам, применяемым к
	конкретным колонкам.

	Для обеспечения совместимости с QSortFilterProxyModel, для выборки используется специальный
	формат строк-фильтров: строки-фильтры для колонок объединены в одну строку, и разделены набором
	символов " | ".
	Например строки "тест", "123", "0.2" должны быть объеденины в строку:
	\code
	"тест | 123 | 0.2"
	\endcode

	ExSortFilterModel также поддерживает выборку данных по диапазону:
	- строка "а - б" приводит к отсеиванию всех значений, не лежащих между "а" и "б" включительно;
	- строка ">а" приводит к отсеиванию всез значений меньших "а";
	- строка "<а" приводит к отсеиванию всез значений больших "а";
	- строка "=а" или "а" приводит к отсеиванию всез значений не равных "а".

	Выборка по диапазону, применима только к чесленным типам и дате-времени(как вместе так и по
	отдельности).

	Формат даты времени: "гггг.ММ.дд чч:мм:сс", где:
	гггг - год полностью
	ММ - порядковый номер месяца в году
	дд - число
	чч - часы
	мм - минуты
	сс - секунды
	* Примечание: все числа меньше 10, должны иметь префикс "0". (т.е. 2008.02.03 04:05:06 а не
	2008.2.3 4:5:6 )

	Пример использования:
	\code
	ExSortFilterModel filter;

	filter->setSourceModel( myModel ); // myModel - любая модель, предположительно табличного формата

	filter->setFilterFixedString( ">1 | 2-3 | 2008.02.03 14:05:00 - 2008.02.05 10:00:00" );
	\endcode

	\sa QSortFilterProxyModel
*/

class ExSortFilterModelPrivate {
	ExSortFilterModel * _t;
public:
	int currentColumn;

	ExSortFilterModelPrivate( ExSortFilterModel * t );

	inline bool processNumber( QString filterstring, double value );
	inline bool processString( QString filterstring, QString value );
	template < typename T >
	inline bool processDateTime( QString filterstring, T value, QString format  );

	inline bool syntaxError();
};

ExSortFilterModelPrivate::ExSortFilterModelPrivate( ExSortFilterModel * t ) {
	_t = t;
	currentColumn = 0;
}

bool ExSortFilterModelPrivate::processNumber( QString filterstring, double value ) {

	bool ok;

	if ( filterstring.contains( " - " ) ) {
		QStringList cap = filterstring.split( " - ", QString::SkipEmptyParts );
		if ( cap.count() != 2 )
			return syntaxError();

		double p_min = cap[0].trimmed().toDouble( &ok );
		if ( !ok )
			return syntaxError();

		double p_max = cap[1].trimmed().toDouble( &ok );
		if ( !ok )
			return syntaxError();


		double max = qMax( p_max, p_min ), min = qMin( p_max, p_min );
		return value >= min && value <= max;
	} else {

		switch ( filterstring.toStdString().c_str()[0] ) {
		case '>':
			filterstring = filterstring.remove( 0, 1 );
			if ( filterstring.isEmpty() )
				return syntaxError();
			{
				double tvalue = filterstring.toDouble( &ok );
				if ( !ok )
					return syntaxError();

				return value > tvalue;
			}
		case '<':
			filterstring = filterstring.remove( 0, 1 );
			if ( filterstring.isEmpty() )
				return syntaxError();
			{
				double tvalue = filterstring.toDouble( &ok );
				if ( !ok )
					return syntaxError();

				return value < tvalue;
			}
		case '=':
			filterstring = filterstring.remove( 0, 1 );
			if ( filterstring.isEmpty() )
				return syntaxError();
			break;
		}

	}

	double tvalue = filterstring.toDouble( &ok );
	if ( !ok )
		return syntaxError();

	return tvalue == value;
}

bool ExSortFilterModelPrivate::processString( QString filterstring, QString value ) {
	return value.contains(filterstring);
}
template < typename T>
bool ExSortFilterModelPrivate::processDateTime( QString filterstring, T value, QString format ) {

	bool ok;

	if ( filterstring.contains( " - " ) ) {
		QStringList cap = filterstring.split( " - ", QString::SkipEmptyParts );
		if ( cap.count() != 2 )
			return syntaxError();

		T p_min = T::fromString( cap[0].trimmed(), format );
		if ( !p_min.isValid() )
			return syntaxError();

		T p_max = T::fromString( cap[1].trimmed(), format );
		if ( !p_max.isValid() )
			return syntaxError();


		T max = qMax( p_max, p_min ), min = qMin( p_max, p_min );
		return value >= min && value <= max;
	} else {

		switch ( filterstring.toStdString().c_str()[0] ) {
		case '>':
			filterstring = filterstring.remove( 0, 1 );
			if ( filterstring.isEmpty() )
				return syntaxError();
			{
				T tvalue = T::fromString( filterstring, format );
				if ( !tvalue.isValid() )
					return syntaxError();

				return value > tvalue;
			}
		case '<':
			filterstring = filterstring.remove( 0, 1 );
			if ( filterstring.isEmpty() )
				return syntaxError();
			{
				T tvalue = T::fromString( filterstring, format );
				if ( !tvalue.isValid() )
					return syntaxError();

				return value < tvalue;
			}
		case '=':
			filterstring = filterstring.remove( 0, 1 );
			if ( filterstring.isEmpty() )
				return syntaxError();
			break;
		}

	}

	T tvalue = T::fromString( filterstring, format );
	if ( !tvalue.isValid() )
		return syntaxError();

	return tvalue == value;
}

bool ExSortFilterModelPrivate::syntaxError() {
	_t->emit_syntaxError(currentColumn);
	return true;
}


ExSortFilterModel::ExSortFilterModel( QObject * parent ) : QSortFilterProxyModel( parent ) {
	_this = this;
}
ExSortFilterModel::~ExSortFilterModel() {
}

bool ExSortFilterModel::rowDataAccept( ExSortFilterModelPrivate & p, QString filterstring, QVariant fielddata ) const {

	switch ( fielddata.type() ) {
	case QVariant::UInt:
	case QVariant::ULongLong:
	case QVariant::Int:
	case QVariant::LongLong:
	case QVariant::Double:
		return p.processNumber( filterstring, fielddata.toDouble() );
	case QVariant::Invalid:
		return true;
	case QVariant::Bool:
		return (filterstring.toInt() == 1) == fielddata.toBool();
	case QVariant::Date:
		return p.processDateTime<QDate>( filterstring, fielddata.toDate(), EXSFM_FORMAT_DATE );
	case QVariant::DateTime:
		return p.processDateTime<QDateTime>( filterstring, fielddata.toDateTime(), EXSFM_FORMAT_DATETIME );
	case QVariant::Time:
		return p.processDateTime<QTime>( filterstring, fielddata.toTime(), EXSFM_FORMAT_TIME );

	/*case QVariant::Char:
	case QVariant::String:*/
	}

	return p.processString( filterstring, fielddata.toString() );
}


bool ExSortFilterModel::filterAcceptsRow( int source_row, const QModelIndex & source_parent ) const {

	ExSortFilterModelPrivate p( _this );

	//qDebug() << "pattern:" << filterRegExp().pattern();

	//int emprtycount = 0;

	foreach ( QString str, filterRegExp().pattern().split( " | " ) ) {
		if ( !str.isEmpty() ) {

			QModelIndex child = sourceModel()->index( source_row, p.currentColumn, source_parent );

			bool accepted = rowDataAccept( p, str, sourceModel()->data( child, Qt::DisplayRole ) );

			if ( !accepted ) {
				return false;
			}
		} /*else
			emprtycount ++;*/
		p.currentColumn++;
	}

	/*if ( p.currentColumn == emprtycount )
		qDebug() << "empty";*/

	return true;
}

void ExSortFilterModel::emit_syntaxError( int columnNo ) {
	emit syntaxError( columnNo );
}
