/***************************************************************************************************
file-encoding:	utf8
doc-generator:	doxygen
author:			dodge
svn-path:		no
***************************************************************************************************/

#include "ModelFilterWidget.h"

#include <QtGui/QLineEdit>
#include <QtGui/QScrollBar>
#include <QtGui/QHeaderView>
#include <QtGui/QAbstractItemView>
#include <QtGui/QSortFilterProxyModel>
#include <QtGui/QTableView>
#include <QtGui/QTreeView>
#include <QtGui/QPainter>
#include <QtCore/QAbstractItemModel>
#include <QtCore/QDebug>

/*!
	\class ModelFilterWidget

	\ingroup Qt Gui extension
	\brief Виджет для отображения редактируемых полей, служащих для фильтрации моделей.
	\mainclass

	В связке с визуализатором моделей, виджет генерирует необходимое количество редактируемых полей
	(QLineEdit). Для нормального функцианированияя виджета, необходим визуализатор, в котором есть
	заголовок (QHeaderView). В библиотеке Qt существует ряд визуализаторов имеющих заголовок.
	На данный момент ModelFilterWidget "умеет" работать с:
	- QTableView
	- QTreeView

	Для того чтобы связать ModelFilterWidget с визуализатором, достаточно вызвать функцию
	setItemView, аргументом которой является указатель на объект визуализатора.

	Пример:
	\code
	ModelFilterWidget filterWidget;
	QTreeView view;

	filterWidget.setItemView( &view );
	...
	\endcode

	\sa QTableView, QTreeView
*/

/*!
	\fn filterChanged( QStringList )

	Сигнал, эмитируемый при вводе пользователем фильтров.
	\a values - список строк-фильтров.
*/

/*!
	\fn filterChanged( QString )

	Сигнал, эмитируемый при вводе пользователем фильтров.
	\a value - строка содержащая все фильтры, в формате: "<фильтр#1> | <фильтр#2>... | <фильтр#n>".
*/

/*!
	Конструктор.
*/
ModelFilterWidget::ModelFilterWidget( QWidget * parent ) : QWidget(parent) {
	p_viewportScrollBar = NULL;
	p_header = NULL;
	additionalheader = NULL;
	p_itemView = NULL;
	setMinimumHeight( 22 );
}
/*!
	Диструктор.
*/
ModelFilterWidget::~ModelFilterWidget() {
}

/*!
	Возвращает указатель на визуализатор, с которым осуществленна последняя связка.

	\sa setItemView( QAbstractItemView * )
*/
QAbstractItemView * ModelFilterWidget::itemView() {
	return p_itemView;
}

/*!
	Возвращает список строк-фильтров.

	\sa filterString()
*/
QStringList ModelFilterWidget::filterStrings() {
	QStringList result;

	foreach ( QLineEdit * le, p_editorList )
		result << le->text();

	return result;
}

/*!
	Возвращает строку, содержащую все фильтры, в формате: "<фильтр#1> | <фильтр#2>... | <фильтр#n>".

	\sa setFilterString( QString * ), filterStrings()
*/
QString ModelFilterWidget::filterString() {
	return filterStrings().join( " | " );
}


/*!
	Возвращает список указателей на поля для редактирования.
*/
QList<QLineEdit*> ModelFilterWidget::editors() {
	return p_editorList;
}

/*public slots:*/
/*!
	Функция задает значения полей ввода. В качестве аргумента \a str принимает строку в формате:
	"<фильтр#1> | <фильтр#2>... | <фильтр#n>".

	\sa filterStrings(), filterString(), setFilterStrings(QStringList)
*/
void ModelFilterWidget::setFilterString( QString str ) {

	setFilterStrings( str.split( " | " ) );

}
/*!
	Функция задает значения полей ввода.

	\sa filterStrings(), filterString(), setFilterString(QString)
*/
void ModelFilterWidget::setFilterStrings( QStringList strl ) {
	int count = strl.count();
	if ( count > p_editorList.count() )
		count = p_editorList.count();

	for ( int i=0; i<count; i++ )
		p_editorList[i]->setText( strl[i] );
}
/*!
	Функция создает связку с визуализатором.

	\sa itemView()
*/
void ModelFilterWidget::setItemView( QAbstractItemView * model ) {

	p_itemView = NULL;

	if ( QTableView * view = qobject_cast<QTableView*>( model ) ) {

		setHeader( view->horizontalHeader() );
		setScrollBar( view->horizontalScrollBar() );

		additionalheader = view->verticalHeader();

		p_itemView = view;
	}

	if ( QTreeView * view = qobject_cast<QTreeView*>( model ) ) {

		setHeader( view->header() );
		setScrollBar( view->horizontalScrollBar() );

		additionalheader = NULL;

		p_itemView = view;
	}

	if ( p_itemView == NULL )
		qWarning( "Warning: model view for ModelFilterWidget is not QTreeView or QTableView subclass" );
	else
		update();

}

/*!
	Функция для принудительного обновления параметров связки (кол-во полей для редактирования,
	размеры полей).
*/
void ModelFilterWidget::update() {
	if ( p_header == NULL )
		return;

	if ( p_header->count() != p_editorList.count() ) {

		while ( p_editorList.count() > 0 )
			delete p_editorList.takeFirst();

		for ( int i=0; i<p_header->count(); i++ ) {

			QLineEdit * le = new QLineEdit( this );
			le->show();

			p_editorList.append( le );

			connect( le, SIGNAL( textChanged(QString) ), this, SLOT( textChanged(QString) ) );

		}
	}

	resizeSections();
	QWidget::update();
}

/*private:*/
void ModelFilterWidget::resizeEvent( QResizeEvent * event ) {
	QWidget::resizeEvent( event );
	resizeSections();
}
void ModelFilterWidget::paintEvent( QPaintEvent * event ) {
	/*if ( p_editorList.count() == 0 ) {
		QPainter painter( this );

		painter.drawLine( 0, 0, rect().width(), rect().height() );
		painter.drawLine( 0, rect().height(), rect().width(), 0 );

		painter.drawText( 0, 20, "<empty>" );
	}*/

	// resizeSections();
	QWidget::paintEvent(event);
}

/*private slots:*/
void ModelFilterWidget::textChanged( QString ) {
	emit filterChanged( filterStrings() );
	emit filterChanged( filterString() );
}

void ModelFilterWidget::resizeSections( int v ) {

	if ( p_itemView == NULL )
		return;

	if ( p_header == NULL )
		return;

	p_itemView->update();
	p_header->update();

	int index = 0, max_width = rect().width(), _left = p_header->offset(), offset = 0;
	if ( additionalheader != NULL )
		offset = additionalheader->isVisible() ? additionalheader->rect().width() : 0 ;
		//offset = additionalheader->rect().width();

	foreach ( QLineEdit * le, p_editorList ) {

		if ( p_header->isSectionHidden( index ) ) {
			le->hide();
		} else {
			le->show();
		}

		int pos = ( p_header->sectionPosition( index )-_left )+offset,
			size = p_header->sectionSize( index );


		le->move( pos, 0 );
		le->resize( size, 22 );

		index++;
	}
	// QWidget::update();
}
void ModelFilterWidget::sectionMoved( int logicalindex, int oldvisualindex, int newvisualindex ) {
	resizeSections();
}
void ModelFilterWidget::sectionResized( int logicalindex, int oldsize, int newsize ) {
	resizeSections();
}
void ModelFilterWidget::sectionCountChanged( int oldcount, int newcount ) {
	if ( p_editorList.count() == newcount )
		return;

	if ( oldcount > newcount )
		return update();

	for ( int i=oldcount; i<newcount; i++ ) {

		QLineEdit * le = new QLineEdit( this );
		le->show();

		p_editorList.append( le );

		connect( le, SIGNAL( textChanged(QString) ), this, SLOT( textChanged(QString) ) );

	}
	resizeSections();
}

void ModelFilterWidget::setHeader( QHeaderView * header ) {
	if ( p_header != NULL ) {
		disconnect( p_header, SIGNAL( sectionMoved(int,int,int) ),
			this, SLOT( sectionMoved(int,int,int) ) );
		disconnect( p_header, SIGNAL( sectionResized(int,int,int) ),
			this, SLOT( sectionResized(int,int,int) ) );
		disconnect( p_header, SIGNAL( sectionCountChanged(int,int) ),
			this, SLOT( sectionCountChanged(int,int) ) );
		p_header = NULL;

		foreach ( QLineEdit * le, p_editorList )
			delete le;
		p_editorList.clear();
	}

	if ( header == NULL )
		return;

	p_header = header;

	connect( p_header, SIGNAL( sectionMoved(int,int,int) ),
		this, SLOT( sectionMoved(int,int,int) ) );
	connect( p_header, SIGNAL( sectionResized(int,int,int) ),
		this, SLOT( sectionResized(int,int,int) ) );
	connect( p_header, SIGNAL( sectionCountChanged(int,int) ),
		this, SLOT( sectionCountChanged(int,int) ) );

}
void ModelFilterWidget::setScrollBar( QScrollBar * scrollbar ) {
	if ( p_viewportScrollBar != NULL ) {
		disconnect( p_viewportScrollBar, SIGNAL( valueChanged(int) ),
			this, SLOT( resizeSections(int) ) );
		p_viewportScrollBar = NULL;
	}

	if ( scrollbar == NULL )
		return;

	p_viewportScrollBar = scrollbar;

	connect( p_viewportScrollBar, SIGNAL( valueChanged(int) ),
		this, SLOT( resizeSections(int) ) );

}
